import org.apache.commons.httpclient.*
import org.apache.commons.httpclient.methods.*;
import org.apache.commons.httpclient.protocol.*;
import org.apache.commons.httpclient.auth.AuthScope;
import com.urbancode.commons.util.https.OpenSSLProtocolSocketFactory;


final def isWindows = (System.getProperty('os.name') =~ /(?i)windows/).find()
final def ahptool = isWindows ? 'ahptool.cmd' : 'ahptool'
final def out = System.out

final def getStepProperties = {
    def props = [:]

    def ahptoolProc = [ahptool, "getStepProperties"].execute()
    ahptoolProc.consumeProcessErrorStream(System.out);
    def propertiesXml = ahptoolProc.text
    ahptoolProc.waitFor()
    if (ahptoolProc.exitValue()) {
        throw new Exception("Failed to get step properties from ahptool: " + propertiesXml)
    }
    new XmlSlurper().parseText(propertiesXml).property.each{ propElem ->
        props[propElem.@name.text()] = propElem.text()
    }

    return props;
}

//-------------------------------------------------------------------------
// Scripts variables
//=========================================================================

final def props = getStepProperties()


String proxyHost = props['integration/proxyHost']
String proxyPort = props['integration/proxyPort']
String proxyPass = props['integration/proxyPass']
String proxyUser = props['integration/proxyUser']
System.out.println "Server:\t"+props['integration/rallyUrl']
System.out.println "Artifact(s):\t"+ props['artifactKey']
System.out.println "Type:\t" + props['type'];
System.out.println "Property:\t"+props['property']
System.out.println "New Value:\t"+ props['newValue']
                                         
if(!props['newValue']) {
	props['newValue'] = "";
} else if(props['newValue'].trim().length() == 0) {
	props['newValue'] = "";
}

String[] newValArray = props['newValue'].split(',');
String[] artifactArray = props['artifactKey'].split(',');
if(newValArray.length != 1 && 
		newValArray.length != artifactArray.length) {
	System.out.println("The number of New Value items must be 1 or match the number of Artifacts!");
	System.exit(1);
}

                                         
if (props['artifactKey'] && props['artifactKey'].trim().length() >0) {
	

    //get rally props for GETS and POSTS to rally
    String base_url = new URL(props['integration/rallyUrl']).getHost()
    String url = props['integration/rallyUrl']+"/slm/webservice/"+props['integration/rallyVer']+"/"
    String user = props['integration/rallyUser']
    String password = props['integration/rallyPass']?props['integration/rallyPass']:props['integration/passScript']

    // create and setup httpClient

    HttpClient client = new HttpClient()
    client.getParams().setAuthenticationPreemptive(true)
    Credentials defaultcreds = new UsernamePasswordCredentials(user, password)
    client.getState().setCredentials(new AuthScope(base_url, 443), defaultcreds)
    
    ProtocolSocketFactory socketFactory = new OpenSSLProtocolSocketFactory()
    Protocol https = new Protocol("https", socketFactory, 443)
    Protocol.registerProtocol("https", https)
    
    // setup proxy if neccessary
    if (proxyHost) {
        client.getHostConfiguration().setProxy(proxyHost, Integer.valueOf(proxyPort));
        if (proxyUser && proxyPass) {
            client.getState().setProxyCredentials(AuthScope.ANY,
                    new UsernamePasswordCredentials(proxyUser, proxyPass));
        }
    }



    def final getWorkspace = {
        HttpMethod workspaceMethod = null
        try {
            workspaceMethod= new GetMethod(url + "subscription")
            workResult = client.executeMethod(workspaceMethod)
            workResponse = workspaceMethod.getResponseBodyAsString()
            workXml = new XmlParser().parseText(workResponse)
            return workXml.Workspaces.Workspace.findAll{it.'@refObjectName'.equals(props['integration/workspace']) }[0].'@ref'
        }
        finally {
           if(workspaceMethod != null) {
               workspaceMethod.releaseConnection()
           }
        }
    }
    workspaceRef = getWorkspace()

    def isError = false;
    for(int i = 0; i<artifactArray.length; i++) {
    	def artifact = artifactArray[i];
    	def newVal = newValArray.length == 1 ? newValArray[0] : newValArray[i];
    	def writer = new StringWriter();
    	def builder = new groovy.xml.MarkupBuilder(writer);
    	def typeNode = builder.createNode(props['type']);
			def propertyNode = builder.createNode(props['property'], newVal);
			builder.nodeCompleted(typeNode, propertyNode);
		builder.nodeCompleted(null, typeNode);
        if (artifact.trim().length() > 0) {
        	//query to get objectId from formattedID
            HttpMethod queryMethod = new GetMethod(url + props['type'] + "?workspace=" + URLEncoder.encode(workspaceRef, "UTF-8") + "&query=" + URLEncoder.encode("(FormattedID = " + artifact.trim() + ")" ,"UTF-8") + "&" + URLEncoder.encode("fetch", "UTF-8") + "=" + URLEncoder.encode("true", "UTF-8"))
            queryResult=client.executeMethod(queryMethod)
            //super cool xml parser to get the objectID of the first result(i love you groovy)
            queryResponse=new XmlParser().parseText(queryMethod.getResponseBodyAsString()).Results.Object.ObjectID[0].text()


            //post to update the object in question
            HttpMethod method = new PostMethod(url+props['type']+"/"+queryResponse)
            method.setRequestEntity((RequestEntity)new StringRequestEntity(writer.toString(), null, null));
            result = client.executeMethod(method)
            response = method.getResponseBodyAsString()


            //check results
            def changeResult = new XmlParser().parseText(response);
            def errors = changeResult.Errors.OperationResultError.size();
            def warnings = changeResult.Warnings.OperationResultWarning.size();

            if(errors == 0 && warnings == 0 && result >=200 && result <300) {
              System.out.println("Successfully updated item " + artifact);
            }
            else {
              System.out.println("ERROR!!! Rally Server could not update item " + artifact);
              System.out.println("Update XML was : " + writer.toString())
              changeResult.Errors.OperationResultError.each { System.out.println(it.text()) }
              changeResult.Warnings.OperationResultWarning.each { System.out.println(it.text()) }
              isError = true
            }
        }
    }
    if(isError) {
    	System.exit(1);
    }
    System.exit(0);
}
else {
	System.out.println 'No artifacts found to update!';
	System.exit(0);
}